// product related settings
// you can change these
const myProductName = "Linky";
const myProductRegKey = "linky";
const myProductRegVersion = "2.7.2.6";
const myJarFileName = "linky.jar";
const myJarFileSize = 114; // the filesize in KB of myJarFileName
const myJSFileName = "linky.js"; // the default prefs file. set it to "" if not used
const myJSFileSize = 2; // the filesize in KB of myJSFileName
const myLocales = new Array("en-US"); // the locales available
const mySkin = new Array(); // the skins available
const myJarCheckDupe = 1; // check if myJarFileName exist both in personal chrome folder and global chrome folder set it to "" if not used
const myJarCheckReq = ""; // are there any requirements for this extension? set it to "" if not used
const myJarCheckReqOS = ""; // are there any OS requirements for this extension? Supported OS is win, mac and unix. set it to "" if not used

// global settings
// dont change these
const sysChromeGlobal = getFolder("Chrome");
const sysChromeGlobalJar = getFolder("Chrome", myJarFileName);
const sysChromeUser = getFolder("Current User", "chrome");
const sysChromeUserJar = getFolder("Current User", "chrome/" + myJarFileName);
const errPage = "http://multizilla.mozdev.org/xpi/return-codes.html";

// OS detection
function getPlatform() {
	var platformStr;
	var platformNode;
	if ("platform" in Install) {
		platformStr = new String(Install.platform);
		if (!platformStr.search(/^Macintosh/))
			platformNode = "mac";
		else if (!platformStr.search(/^Win/))
			platformNode = "win";
		else if (!platformStr.search(/^OS\/2/))
			platformNode = "win";
		else
			platformNode = "unix";
	} else {
		var fOSMac	= getFolder("Mac System");
		var fOSWin	= getFolder("Win System");
		if (fOSMac != null)
			platformNode = "mac";
		else if(fOSWin != null)
			platformNode = "win";
		else
			platformNode = "unix";
	}
	return platformNode;
}

// verify disk space in kilobytes
function verifyDiskSpace(dirPath, spaceRequired) {
	var spaceAvailable = fileGetDiskSpaceAvailable(dirPath);
	if (parseInt(spaceAvailable/1024) < spaceRequired) {
		logComment("Insufficient disk space: " + dirPath + "\n\trequired : " + spaceRequired + " K\n\tavailable: " + spaceAvailable + " K");
		return false;
	} else {
		return true;
	}
}

var oci = false; // one-click-install
var inst; // inst is the installation method:
//		0 means that the installation was cancelled.
//		1 is in the global chrome folder. This *has* to be the default!
//		2 in in the personal profile folder. This is only supported for newer builds.

if (myJarCheckDupe && File.exists(sysChromeGlobalJar) && File.exists(sysChromeUserJar)) {
	alert(myProductName + " is installed in BOTH the personal chrome folder:\n" + sysChromeUserJar + "\nand the global chrome folder:\n" + sysChromeGlobalJar + "\n\nThis will cause weird behaviour. Please delete one of the instances!");
	inst = 0;
} else {
	if (myJarCheckReq) {
		for (var i = 0; i < myJarCheckReq.length; i++) {
			if (!File.isDirectory(getFolder("Chrome", myJarCheckReq[i])) && !File.isDirectory(getFolder("Current User", "chrome/" + myJarCheckReq[i])) && !File.exists(getFolder("Chrome", myJarCheckReq[i] + ".jar")) && !File.exists(getFolder("Current User", "chrome/" + myJarCheckReq[i] + ".jar"))) {
				alert(myProductName + " cant be installed since " + myJarCheckReq[i] + " needs to be installed before installing " + myProductName + "\nPlease visit mozdev.org and install the " + myJarCheckReq[i] + " module first");
				inst = 0;
			}
		}
	}
	if (myJarCheckReqOS) {
		var os = getPlatform();
		if (os && os != myJarCheckReqOS) {
			alert(myProductName + " cant be installed since your operating system (" + os + ") needs to be " + myJarCheckReqOS);
			inst = 0;
		}
	}

	if (typeof(inst) != "number") {
		var ans;
		ans = confirm(myProductName + " version " + myProductRegVersion + " installation.\nClick OK to use one-click-install or click Cancel to choose yourself where to install.");
		if (ans)
			oci = true;

		// Can we install in the personal profile folder? Assumes that builds without a buildID are used by people who knows what they are doing.
		if (!buildID || (buildID && buildID >= 2003030700)) {
			if (!oci)
				alert("You can install " + myProductName + " " + myProductRegVersion + " in two ways:\n\n1) in the global chrome folder. This will make " + myProductName + " available in all profiles.\n\n2) in the personal profile folder. This will make " + myProductName + " available only in the current profile.\n\nYou will now be prompted to select a way...");

			// Check if extension already is installed
			if (File.exists(sysChromeGlobalJar)) {
				if (!oci)
					ans = confirm("A version of " + myProductName + " is already installed in the global chrome folder:\n" + sysChromeGlobal + "\nIt's recommended that you install " + myProductName + " into this folder.\nClick OK to install into this folder.");
				inst = 1;
			} else if (File.exists(sysChromeUserJar)) {
				if (!oci)
					ans = confirm("A version of " + myProductName + " is already installed in the personal chrome folder:\n" + sysChromeUser + "\nIt's recommended that you install " + myProductName + " into this folder.\nClick OK to install into this folder.");
				inst = 2;
			} else {
				inst = 2;
			}

			if (!ans && !oci) {
				if (inst != 1)
					ans = confirm("Click OK to install in the global chrome folder:\n" + sysChromeGlobal + "\nThis is the recommended install option!");
				if (!ans) {
					if (inst != 1)
						ans = confirm("Click OK to install in your personal profile folder:\n" + sysChromeUser);
					if (!ans) {
						alert("You didn't choose any of the supported installation options!\nPlease restart the installation!");
						inst = 0;
					} else {
						inst = 2;
					}
				} else {
					inst = 1;
				}
			}
		} else {
			if (!oci) {
				ans = confirm("Click OK to install " + myProductName + " " + myProductRegVersion + " in the global chrome folder:\n" + sysChromeGlobal);
				if (!ans) {
					alert("You didn't choose any of the supported installation options!\nPlease restart the installation!");
					inst = 0;
				} else {
					inst = 1;
				}
			} else {
				inst = 1;
			}
		}
	}
}

// did the user cancel?
if (inst) {
	var err;
	var err_tmp;

	// init the installation
	err_tmp = initInstall(myProductName, myProductRegKey, myProductRegVersion);
	if (err_tmp) {
		err = err_tmp;
	}

	// what folder to install into
	var folder = (inst == 1 ? sysChromeGlobal : sysChromeUser);

	if (buildID)
		logComment(myProductName + " version <" + myProductRegVersion + "> being installed on build <" + buildID + ">");
	else
		logComment(myProductName + " version <" + myProductRegVersion + "> being installed on a build without buildID");
	logComment("Installation method is " + inst);

	// check if there"s disk space for myJSFileSize
	if (!err && myJSFileName && !verifyDiskSpace(sysChromeGlobal, myJSFileSize)) {
		err = INSUFFICIENT_DISK_SPACE;
	}

	// check if there"s disk space for myJarFileSize
	if (!err && myJarFileName && !verifyDiskSpace(folder, myJarFileSize)) {
		err = INSUFFICIENT_DISK_SPACE;
	}

	// add the jar file
	if (!err) {
		err_tmp = addFile(myProductName, "chrome/" + myJarFileName, folder, null);
		if (err_tmp) {
			logComment("Problem adding jar file <" + myJarFileName + "> to <" + folder + ">. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			logComment("ok adding jar file <" + myJarFileName + "> to <" + folder + ">.");
		}
	}

	// add the default prefs, but only if we have to install into global chrome
	if (!err && inst == 1 && myJSFileName) {
		err_tmp = addFile(myProductName, "defaults/preferences/" + myJSFileName, getFolder("Program","defaults/pref"), null);
		if (err_tmp) {
			logComment("Problem adding default prefs <" + myJSFileName + "> to <" + getFolder("Program", "defaults/pref") + ">. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			logComment("ok adding default prefs <" + myJSFileName + "> to <" + getFolder("Program","defaults/pref") + ">.");
		}
	}

	// register the content
	if (!err) {
		folder = getFolder(folder, myJarFileName);
		if (inst == 2) {
			err_tmp = registerChrome(CONTENT | PROFILE_CHROME, folder, "content/");
		} else {
			err_tmp = registerChrome(CONTENT | DELAYED_CHROME, folder, "content/");
		}
		if (err_tmp) {
			logComment("Problem registering the content chrome. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			logComment("ok registering the content chrome.");
		}
	}

	// register the locale
	if (!err) {
		if (inst == 2) {
			for (var i = 0; i < myLocales.length; i++) {
				err_tmp = registerChrome(LOCALE | PROFILE_CHROME, folder, "locale/" + myLocales[i] + "/");
				if (err_tmp)
					break;
			}
		} else {
			for (var i = 0; i < myLocales.length; i++) {
				err_tmp = registerChrome(LOCALE | DELAYED_CHROME, folder, "locale/" + myLocales[i] + "/");
				if (err_tmp) {
					break;
				}
			}
		}
		if (err_tmp) {
			logComment("Problem registering the locale. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			logComment("ok registering the locale.");
		}
	}

	// register the skin
	if (!err) {
		if (inst == 2) {
			for (var i = 0; i < mySkin.length; i++) {
				err_tmp = registerChrome(SKIN | PROFILE_CHROME, folder, "skin/" + mySkin[i] + "/");
				if (err_tmp)
					break;
			}
		} else {
			for (var i = 0; i < mySkin.length; i++) {
				err_tmp = registerChrome(SKIN | DELAYED_CHROME, folder, "skin/" + mySkin[i] + "/");
				if (err_tmp) {
					break;
				}
			}
		}
		if (err_tmp) {
			logComment("Problem registering the skin. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			logComment("ok registering the skin.");
		}
	}

	// do the install
	if (!err) {
		err_tmp = performInstall();
		if (err_tmp < 0) {
			logComment("Problem performing install. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			if (!oci)
				alert(myProductName + " version " + myProductRegVersion + " has been successfully installed!\nWhen you right-click on a webpage your will now have a new context menuitem called " + myProductName + "!\n\nYou must restart your browser to complete installation!");
		}
	}

	// did it work?
	if (err) {
		logComment("Problem installing. Error code: " + err + ". Error codes can been seen at: " + errPage);
		alert(myProductName + " version " + myProductRegVersion + " was not installed!\nError code: " + err + "\n\nError codes can been seen at:\n" + errPage)
		cancelInstall(err);
	}
} else {
	cancelInstall();
}
